Package: gmp
Version: 0.6-2
Date: 2021-01-07
Title: Multiple Precision Arithmetic
Author: Antoine Lucas, Immanuel Scholz, Rainer Boehme <rb-gmp@reflex-studio.de>,
  Sylvain Jasson <Sylvain.Jasson@inrae.fr>,
  Martin Maechler <maechler@stat.math.ethz.ch>
Maintainer: Antoine Lucas <antoinelucas@gmail.com>
Description: Multiple Precision Arithmetic (big integers and rationals,
 prime number tests, matrix computation), "arithmetic without limitations"
 using the C library GMP (GNU Multiple Precision Arithmetic).
Depends: R (>= 3.5.0)
Imports: methods
Suggests: Rmpfr, MASS, round
SystemRequirements: gmp (>= 4.2.3)
License: GPL (>= 2)
BuildResaveData: no
LazyDataNote: not available, as we use data/*.R *and* our classes
NeedsCompilation: yes
URL: https://forgemia.inra.fr/sylvain.jasson/gmp
Packaged: 2021-01-07 13:01:36 UTC; antoine
Repository: CRAN
Date/Publication: 2021-01-07 14:10:06 UTC
Built: R 3.6.3; x86_64-w64-mingw32; 2021-01-07 17:14:15 UTC; windows
Archs: x64
