Package: gmp
Version: 0.5-13.2
Date: 2018-07-14
Title: Multiple Precision Arithmetic
Author: Antoine Lucas, Immanuel Scholz, Rainer Boehme <rb-gmp@reflex-studio.de>,
  Sylvain Jasson <jasson@toulouse.inra.fr>,
  Martin Maechler <maechler@stat.math.ethz.ch>
Maintainer: Antoine Lucas <antoinelucas@gmail.com>
Description: Multiple Precision Arithmetic (big integers and rationals,
 prime number tests, matrix computation), "arithmetic without limitations"
 using the C library GMP (GNU Multiple Precision Arithmetic).
Depends: R (>= 3.0.0)
Imports: methods
Suggests: Rmpfr
SystemRequirements: gmp (>= 4.2.3)
License: GPL
BuildResaveData: no
LazyDataNote: not available, as we use data/*.R *and* our classes
NeedsCompilation: yes
Packaged: 2018-07-14 11:35:43 UTC; ripley
Repository: CRAN
Date/Publication: 2018-07-14 14:19:11 UTC
Built: R 3.5.1; x86_64-w64-mingw32; 2018-10-11 01:46:41 UTC; windows
Archs: x64
