# RFC 2409 Diffie Hellman Key Exchange Groups (standardized generators and primes)

Oakley1 <- NULL

if(requireNamespace("gmp", quietly=TRUE)) 
  Oakley1 <- gmp::as.bigz(2,
  "0xFFFFFFFFFFFFFFFFC90FDAA22168C234C4C6628B80DC1CD129024E088A67CC74020BBEA63B139B22514A08798E3404DDEF9519B3CD3A431B302B0A6DF25F14374FE1356D6D51C245E485B576625E7EC6F44C42E9A63A3620FFFFFFFFFFFFFFFF") else
warning("Data for RFC 2409 Oakley groups requires package 'gmp'")  
